# alternative implementation for retrieving bin data from Kingston Upon Thames Council
# principal URL is https://waste-services.kingston.gov.uk/waste/[uprn]
# https://www.kingston.gov.uk/info/200287/bins_and_recycling/1113/check_your_bin_collection_day

# switched to using Selenium as the htmx elements are not rendered reliably with requests

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
import re

# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        driver = None
        try:

            headless = kwargs.get("headless")
            web_driver = kwargs.get("web_driver")
            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))
            wait = WebDriverWait(driver, 15, 2)

            wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "waste-service-name"))
            )

            data = {"bins": []}

            soup = BeautifulSoup(driver.page_source, "html.parser")
            collections = soup.find_all("h3", {"class": "waste-service-name"})
            for c in collections:
                rows = c.find_next_sibling("div", {"class": "govuk-grid-row"}).find_all(
                    "div", {"class": "govuk-summary-list__row"}
                )
                for row in rows:
                    if row.find("dt").get_text().strip().lower() == "next collection":
                        collection_date = remove_ordinal_indicator_from_date_string(
                            row.find("dd").get_text()
                        ).strip()
                        # strip out any text inside of the date string
                        collection_date = re.sub(r'\n\s*\(this.*?\)', '', collection_date)
                        dict_data = {
                            "type": c.get_text().strip().capitalize(),
                            "collectionDate": get_next_occurrence_from_day_month(
                                datetime.strptime(
                                    collection_date + " " + datetime.now().strftime("%Y"),
                                    "%A, %d %B %Y",
                                )
                            ).strftime(date_format),
                        }
                        data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()

        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        uprn = kwargs.get("uprn")
        check_uprn(uprn)  # Assuming check_uprn() raises an exception if UPRN is invalid

        try:
            response = requests.post(
                f"https://wastecollections.haringey.gov.uk/property/{uprn}",
                timeout=10  # Set a timeout for the request
            )
            response.raise_for_status()  # This will raise an exception for HTTP errors
        except requests.RequestException as e:
            logging.error(f"Network or HTTP error occurred: {e}")
            raise ConnectionError("Failed to retrieve data.") from e

        try:
            soup = BeautifulSoup(response.text, features="html.parser")
            soup.prettify()

            sections = soup.find_all("div", {"class": "property-service-wrapper"})

            date_regex = re.compile(r"\d{2}/\d{2}/\d{4}")
            for section in sections:
                service_name_element = section.find("h3", {"class": "service-name"})
                next_service_element = section.find("tbody").find("td", {"class": "next-service"})

                if service_name_element and next_service_element:
                    service = service_name_element.text
                    next_collection = next_service_element.find(text=date_regex)

                    if next_collection:
                        dict_data = {
                            "type": service.replace("Collect ", "").replace("Paid ", "").strip(),
                            "collectionDate": next_collection.strip(),
                        }
                        data["bins"].append(dict_data)
        except Exception as e:
            logging.error(f"Error parsing data: {e}")
            raise ValueError("Error processing the HTML data.") from e

        return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            uprn = kwargs.get("uprn")
            postcode = kwargs.get("postcode")
            full_address = kwargs.get("paon")

            url = "https://my.guildford.gov.uk/customers/s/view-bin-collections"

            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 120)
            post_code_search = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//input[contains(@class, 'slds-input')]")
                )
            )

            post_code_search.send_keys(postcode)

            post_code_submit_btn = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//button[contains(@class,'slds-button')]")
                )
            )
            post_code_submit_btn.send_keys(Keys.ENTER)

            # Locate the element containing the specified address text
            address_element = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        f"//lightning-base-formatted-text[contains(text(), '{full_address}')]",
                    )
                )
            )

            # Find the associated radio button in the same row (preceding sibling)
            radio_button = address_element.find_element(
                By.XPATH, "../../../../preceding-sibling::td//input[@type='radio']"
            )

            radio_button.send_keys(Keys.SPACE)
            address_submit_btn = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//button[contains(@name,'NEXT')]")
                )
            )
            address_submit_btn.send_keys(Keys.ENTER)

            results = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "cBinScheduleDisplay"))
            )

            results2 = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f'//div[contains(@title,"Bin Job")]')
                )
            )
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            # Find all table rows containing bin information
            rows = soup.find_all("tr", class_="slds-hint-parent")

            data = {"bins": []}

            # Extract bin type and next collection date for each row
            for row in rows:
                bin_type = (
                    row.find("td", {"data-label": "Bin Job"})
                    .find("strong")
                    .text.strip()
                    if row.find("td", {"data-label": "Bin Job"})
                    else None
                )

                next_collection_date = (
                    row.find("td", {"data-label": "Next Collection"}).text.strip()
                    if row.find("td", {"data-label": "Next Collection"})
                    else None
                )

                if bin_type and next_collection_date:
                    # Convert date string to datetime object
                    date_format = (
                        "%A, %d %B"  # Adjust the format according to your date string
                    )
                    try:
                        next_collection_date = datetime.strptime(
                            next_collection_date, date_format
                        )

                        # Logic to determine year
                        current_date = datetime.now()
                        if next_collection_date.month < current_date.month:
                            year = current_date.year + 1
                        else:
                            year = current_date.year

                        # Format the date
                        next_collection_date = next_collection_date.replace(
                            year=year
                        ).strftime("%d/%m/%Y")
                    except ValueError:
                        pass

                    dict_data = {
                        "type": bin_type,
                        "collectionDate": next_collection_date,
                    }
                    data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_uprn = kwargs.get("uprn")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            # Pad UPRN with 0's at the start for any that aren't 12 chars
            user_uprn = user_uprn.zfill(12)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(
                f"https://my.reigate-banstead.gov.uk/en/service/Bins_and_recycling___collections_calendar?uprn={user_uprn}"
            )

            # Wait for iframe to load and switch to it
            WebDriverWait(driver, 30).until(
                EC.frame_to_be_available_and_switch_to_it((By.ID, "fillform-frame-1"))
            )

            # Wait for form
            WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, 'span[data-name="html2"] > div')
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            data = {"bins": []}
            section = soup.find("span", {"data-name": "html2"})
            dates = section.find_all("div")
            for d in dates:
                date = d.find("h3")
                collections = d.find_all("li")
                if date and collections:
                    collection_date = datetime.strptime(
                        date.get_text(strip=True), "%A %d %B %Y"
                    ).strftime(date_format)
                    for c in collections:
                        collection_type = c.get_text(strip=True)
                        if c.get_text(strip=True):
                            dict_data = {
                                "type": collection_type,
                                "collectionDate": collection_date,
                            }
                            data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from selenium.webdriver.common.keys import Keys


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://selfservice.preston.gov.uk/service/Forms/FindMyNearest.aspx?Service=bins"

            data = {"bins": []}

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property
            inputElement_address = driver.find_element(
                By.ID,
                "MainContent_txtAddress",
            )

            inputElement_address.send_keys(user_paon)
            inputElement_address.send_keys(" ")
            inputElement_address.send_keys(user_postcode)

            driver.find_element(
                By.ID,
                "btnSearch",
            ).send_keys(Keys.ENTER)

            # Wait for the 'Select your property' dropdown to appear and select the first result
            dropdown = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "MainContent_ddlSearchResults"))
            )
            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_index(1)

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            topLevelSpan = soup.find("span", id="MainContent_lblMoreCollectionDates")

            collectionDivs = topLevelSpan.find_all("div", {"id": "container"})

            for collectionDiv in collectionDivs:
                type_and_date_divs = collectionDiv.find_all("b")
                bin_type = type_and_date_divs[0].text

                date_elements = collectionDiv.find_all("li")
                for date_element in date_elements:
                    date_string = date_element.find("span").text.split(" ")[1]
                    collection_date = datetime.strptime(
                        date_string, "%d/%m/%Y"
                    ).strftime(date_format)

                    data["bins"].append(
                        {
                            "type": re.sub(r"[^a-zA-Z0-9,\s]", "", bin_type).strip(),
                            "collectionDate": collection_date,
                        }
                    )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests
import json
import urllib.parse
from dateutil.relativedelta import relativedelta
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        data = {"bins": []}

        headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Language': 'en-GB,en;q=0.9',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            # Already added when you pass json=
            # 'Content-Type': 'application/json',
            # 'Cookie': 'ASP.NET_SessionId=n2kxv5ssap4gobb11va1oxge',
            'Origin': 'https://tdcws01.tandridge.gov.uk',
            'Pragma': 'no-cache',
            'Referer': 'https://tdcws01.tandridge.gov.uk/TDCWebAppsPublic/tfaBranded/408?utm_source=pressrelease&utm_medium=smposts&utm_campaign=check_my_bin_day',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/121.0.6167.186 Safari/537.36',
        }

        params = {
            'UPRN': f'{user_uprn}',
        }

        json_data = requests.post(
            'https://tdcws01.tandridge.gov.uk/TDCWebAppsPublic/TDCMiddleware/RESTAPI/WhiteSpaceAPI/GetCompleteRecordByUPRN',
            headers=headers,
            json=params,
        ).json()['lstNextCollections']


        for item in json_data:
            dict_data = {
                "type": item.get('Service').replace('Collection Service', '').strip(),
                "collectionDate": datetime.strptime(item.get('Date'), "%d/%m/%Y %H:%M:%S").strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import urllib3
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # get the page data
        request = urllib3.request(method="get", url=kwargs['url'])
        page_data = request.data

        # Make a BS4 object
        soup = BeautifulSoup(page_data, features="html.parser")
        soup.prettify()

        # Form a JSON wrapper
        data = {"bins": []}

        # Find section with bins in
        sections = (
            soup.find_all("div", {"class": "card h-100"})
        )

        # there may also be a recycling one too
        sections_recycling = (
            soup.find_all("div", {"class": "card h-100 card-recycling"})
        )
        if len(sections_recycling) > 0:
            sections.append(sections_recycling[0])

        # For each bin section, get the text and the list elements
        for item in sections:
            header = item.find("div", {"class": "card-header"})
            bin_type_element = header.find_next("b")
            if bin_type_element is not None:
                bin_type = bin_type_element.text
                array_expected_types = ["Domestic", "Recycling"]
                if bin_type in array_expected_types:
                    date = (
                        item.find_next("p", {"class": "card-text"})
                            .find_next("mark")
                            .next_sibling.strip()
                    )
                    next_collection = datetime.strptime(date, "%d/%m/%Y")

                    dict_data = {
                        "type": bin_type,
                        "collectionDate": next_collection.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

        return data



# This script pulls (in one hit) the data
# from Warick District Council Bins Data

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):

    """
    Concrete classes have to implement all abstract operations of the
    baseclass. They can also override some
    operations with a default implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for element in soup.find_all("strong"):
            bin_type = element.next_element
            bin_type = bin_type.lstrip()
            collectionDateElement = element.next_sibling.next_element.next_element
            collectionDate = collectionDateElement.getText()
            dict_data = {
                "type": bin_type,
                "collectionDate": collectionDate,
            }
            data["bins"].append(dict_data)

        return data



import time

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(
                "https://knowsleytransaction.mendixcloud.com/link/youarebeingredirected?target=bincollectioninformation"
            )

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.visibility_of_element_located(
                    (By.XPATH, '/html/body/div[1]/div/div/div/div/div/div[2]/div/div/div/div/div/div[3]/div/div[1]/div/div[1]/div/div/input')
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Wait for address search button, then click it
            addressSearch_button = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, '/html/body/div[1]/div/div/div/div/div/div[2]/div/div/div/div/div/div[3]/div/div[1]/div/div[2]/div/button')
                )
            )
            addressSearch_button.click()

            # Wait until the address list has loaded
            WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.XPATH, '/html/body/div[1]/div/div/div/div/div/div[2]/div/div/div/div/div/div[3]/div/div[1]/div/div[3]/div/div')
                )
            )

            # Select the correct address from the list
            addressList_rows = driver.find_elements(By.CLASS_NAME, 'row')
            for row in addressList_rows:
                option_name = row.text[0:len(user_paon)]
                if option_name == user_paon:
                    break
            address_to_select = row.find_element(By.LINK_TEXT, "Choose this address")
            address_to_select.click()

            # Wait for bin dates to load
            WebDriverWait(driver, 20).until(
                EC.presence_of_element_located(
                    (By.XPATH, '/html/body/div[1]/div/div/div/div/div/div[2]/div/div/div/div/div/div[3]/div/div[1]/div/div[4]/div/div')
                )
            )

            # Parse the HTML from the WebDriver
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            z = soup.find("div", {"class": "mx-name-textBox5 mx-textbox form-group"}).find_next(
                    "div", {"class": "form-control-static"})

            maroon_bin_date = datetime.strptime(
                soup.find("div", {"class": "mx-name-textBox3 mx-textbox form-group"}).find_next(
                    "div", {"class": "form-control-static"}).get_text(strip=True), "%A %d/%m/%Y")
            collections.append(("Maroon bin", maroon_bin_date))

            grey_bin_date = datetime.strptime(
                soup.find("div", {"class": "mx-name-textBox4 mx-textbox form-group"}).find_next(
                    "div", {"class": "form-control-static"}).get_text(strip=True), "%A %d/%m/%Y")
            collections.append(("Grey bin", grey_bin_date))

            blue_bin_date = datetime.strptime(
                soup.find("div", {"class": "mx-name-textBox5 mx-textbox form-group"}).find_next(
                    "div", {"class": "form-control-static"}).get_text(strip=True), "%A %d/%m/%Y")
            collections.append(("Blue bin", blue_bin_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            for item in ordered_data:
                dict_data = {
                    "type": item[0].capitalize(),
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()

        return data



from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        # Get UPRN and postcode from commandline
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        check_uprn(user_uprn)

        # Pass in form data and make the POST request
        headers = {
            "authority": "waste.barnsley.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "accept-language": "en-GB,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded",
            "origin": "https://waste.barnsley.gov.uk",
            "pragma": "no-cache",
            "referer": "https://waste.barnsley.gov.uk/ViewCollection/SelectAddress",
            "sec-ch-ua": '"Chromium";v="118", "Opera GX";v="104", "Not=A?Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.5993.118 Safari/537.36",
        }
        form_data = {
            "personInfo.person1.HouseNumberOrName": "",
            "personInfo.person1.Postcode": f"{user_postcode}",
            "personInfo.person1.UPRN": f"{user_uprn}",
            "person1_SelectAddress": "Select address",
        }
        response = requests.post(
            "https://waste.barnsley.gov.uk/ViewCollection/SelectAddress",
            headers=headers,
            data=form_data,
        )
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        if response.status_code != 200:
            raise ConnectionRefusedError(
                "Error getting results from website! Please open an issue on GitHub!"
            )

        # Parse the response, getting the top box first and then tabled collections after
        results = soup.find("div", {"class": "panel"}).find_all("fieldset")[0:2]
        heading = results[0].find_all("p")[1:3]
        bin_text = heading[1].text.strip() + " bin"

        if heading[0].text == "Today":
            bin_date = datetime.today()
        elif heading[0].text == "Tomorrow":
            bin_date = datetime.today() + relativedelta(days=1)
        else:
            bin_date = datetime.strptime(heading[0].text, "%A, %B %d, %Y")

        dict_data = {
            "type": bin_text,
            "collectionDate": bin_date.strftime(date_format),
        }
        data["bins"].append(dict_data)

        results_table = [row for row in results[1].find_all("tbody")[0] if row != "\n"]
        for row in results_table:
            text_list = [item.text.strip() for item in row.contents if item != "\n"]
            bin_text = text_list[1] + " bin"

            dict_data = {
                "type": bin_text,
                "collectionDate": bin_date.strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Disable the SSL warnings that otherwise break everything
        requests.packages.urllib3.disable_warnings()
        try:
            requests.packages.urllib3.contrib.pyopenssl.util.ssl_.DEFAULT_CIPHERS += (
                ":HIGH:!DH:!aNULL"
            )
        except AttributeError:
            pass

        # UPRN is street id here
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        post_url = "https://apps.castlepoint.gov.uk/cpapps/index.cfm?fa=wastecalendar.displayDetails"
        post_header_str = (
            "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,"
            "image/apng,"
            "*/*;q=0.8,application/signed-exchange;v=b3;q=0.9|Accept-Encoding: gzip, deflate, "
            "br|Accept-Language: en-GB;q=0.8|Cache-Control: max-age=0|Connection: "
            "keep-alive|Content-Length: "
            "11|Content-Type: application/x-www-form-urlencoded|Host: apps.castlepoint.gov.uk|Origin: "
            "https://apps.castlepoint.gov.uk|Referer: "
            "https://apps.castlepoint.gov.uk/cpapps/index.cfm?fa=wastecalendar|Sec-Fetch-Dest: "
            "document|Sec-Fetch-Mode: navigate|Sec-Fetch-Site: same-origin|Sec-Fetch-User: ?1|Sec-GPC: "
            "1|Upgrade-Insecure-Requests: 1|User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.5060.134 Safari/537.36 "
        )

        post_headers = parse_header(post_header_str)
        form_data = {"roadID": uprn}
        post_response = requests.post(
            post_url, headers=post_headers, data=form_data, verify=False
        )

        # Make a BS4 object
        soup = BeautifulSoup(post_response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        collection_tuple = []

        for i in range(1, 3):
            calendar = soup.select(
                f"#wasteCalendarContainer > div:nth-child(2) > div:nth-child({i}) > div"
            )[0]
            month = datetime.strptime(
                calendar.find_next("h2").get_text(), "%B %Y"
            ).strftime("%m")
            year = datetime.strptime(
                calendar.find_next("h2").get_text(), "%B %Y"
            ).strftime("%Y")

            pink_days = [
                day.get_text().strip() for day in calendar.find_all("td", class_="pink")
            ]
            black_days = [
                day.get_text().strip()
                for day in calendar.find_all("td", class_="normal")
            ]

            for day in pink_days:
                collection_date = datetime(
                    year=int(year), month=int(month), day=int(day)
                )
                collection_tuple.append(("Pink collection", collection_date))

            for day in black_days:
                collection_date = datetime(
                    year=int(year), month=int(month), day=int(day)
                )
                collection_tuple.append(("Normal collection", collection_date))

        ordered_data = sorted(collection_tuple, key=lambda x: x[1])

        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://mybexley.bexley.gov.uk/service/When_is_my_collection_day"

            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )

            driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)
            start_btn = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//button/span[contains(text(), 'Next')]")
                )
            )

            start_btn.click()

            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.ID, "postcode_search"))
            )
            inputElement_postcodesearch.send_keys(user_postcode)

            find_address_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//*[@id="search"]'))
            )
            find_address_btn.click()

            dropdown_options = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="select2-chosen-1"]')
                )
            )
            time.sleep(2)
            dropdown_options.click()
            time.sleep(1)
            dropdown_input = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="s2id_autogen1_search"]')
                )
            )
            time.sleep(1)
            dropdown_input.send_keys(user_paon)
            dropdown_input.send_keys(Keys.ENTER)

            results_found = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "found-content"))
            )
            finish_btn = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//button/span[contains(text(), 'Next')]")
                )
            )
            finish_btn.click()
            final_page = wait.until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "waste-header-container")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            bin_fields = soup.find_all("div", class_="waste-panel-container")
            # Define your XPath

            for bin in bin_fields:
                # Extract h3 text from the current element
                h3_text = (
                    bin.find("h3", class_="container-name").get_text(strip=True)
                    if bin.find("h3", class_="container-name")
                    else None
                )

                date_text = (
                    bin.find("p", class_="container-status").get_text(strip=True)
                    if bin.find("p", class_="container-status")
                    else None
                )

                if h3_text and date_text:
                    # Parse the date using the appropriate format
                    parsed_date = datetime.strptime(date_text, "%A %d %B")

                    # Assuming the current year is used for the collection date
                    current_year = datetime.now().year

                    # If the parsed date is in the past, assume it's for the next year
                    if parsed_date < datetime.now():
                        current_year += 1

                    data["bins"].append(
                        {
                            "type": h3_text,
                            "collectionDate": parsed_date.replace(
                                year=current_year
                            ).strftime("%d/%m/%Y"),
                        }
                    )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import json
from unittest import mock

import pytest
from requests import exceptions as req_exp
from requests.models import Response
from uk_bin_collection.get_bin_data import AbstractGetBinDataClass as agbdc
from uk_bin_collection.get_bin_data import setup_logging


def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code, raise_error_type):
            self.text = json_data
            self.status_code = status_code
            if raise_error_type is not None:
                self.raise_for_status = self.raise_error(raise_error_type)
            else:
                self.raise_for_status = lambda: None

        def raise_error(self, errorType):
            if errorType == "HTTPError":
                raise req_exp.HTTPError()
            elif errorType == "ConnectionError":
                raise req_exp.ConnectionError()
            elif errorType == "Timeout":
                raise req_exp.Timeout()
            elif errorType == "RequestException":
                raise req_exp.RequestException()
            return errorType

    if args[0] == "aurl":
        return MockResponse({"test_data": "test"}, 200, None)
    elif args[0] == "HTTPError":
        return MockResponse({}, 999, "HTTPError")
    elif args[0] == "ConnectionError":
        return MockResponse({}, 999, "ConnectionError")
    elif args[0] == "Timeout":
        return MockResponse({}, 999, "Timeout")
    elif args[0] == "RequestException":
        return MockResponse({}, 999, "RequestException")
    elif args[0] == "notPage":
        return MockResponse("not json", 200, None)
    return MockResponse(None, 404, "HTTPError")


# Unit tests


def test_logging_exception():
    logging_dict = "SW1A 1AA"
    with pytest.raises(ValueError) as exc_info:
        result = setup_logging(logging_dict, "ROOT")
    assert exc_info.typename == "ValueError"


@mock.patch("requests.get", side_effect=mocked_requests_get)
def test_get_data(mock_get):
    page_data = agbdc.get_data("aurl")
    assert page_data.text == {"test_data": "test"}


@pytest.mark.parametrize(
    "url", ["HTTPError", "ConnectionError", "Timeout", "RequestException"]
)
@mock.patch("requests.get", side_effect=mocked_requests_get)
def test_get_data_error(mock_get, url):
    with pytest.raises(Exception) as exc_info:
        result = agbdc.get_data(url)
    assert exc_info.typename == url


def test_output_json():
    bin_data = {"bin": ""}
    output = agbdc.output_json(bin_data)
    assert type(output) == str
    assert output == '{\n    "bin": ""\n}'



import re
from datetime import datetime

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def get_token(res) -> str:
    """
    Get a UFPRT code for the form data to be processed
        :param res:
        :return:
    """
    soup = BeautifulSoup(res, features="html.parser")
    soup.prettify()
    token = soup.find("input", {"name": "ufprt"}).get("value")
    return token


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://www.midsussex.gov.uk/waste-recycling/bin-collection/"
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        postcode_re = "^([A-Za-z][A-Ha-hJ-Yj-y]?[0-9][A-Za-z0-9]? ?[0-9][A-Za-z]{2}|[Gg][Ii][Rr] ?0[Aa]{2})$"
        user_full_addr = f"{user_paon} {user_postcode}"

        check_postcode(user_postcode)
        check_paon(user_paon)

        form_data = {
            "PostCodeStep.strAddressSearch": user_postcode,
            "AddressStep.strAddressSelect": user_full_addr,
            "Next": "true",
            "StepIndex": "1",
        }

        # Get a ufprt by posting here (I have no idea how ufprt works, so may as well grab one from the server)
        requests.packages.urllib3.disable_warnings()
        init = requests.post(api_url, data=form_data)
        ufprt = get_token(init.text)
        form_data.update({"ufprt": ufprt})

        response = requests.post(api_url, data=form_data)

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        table_element = soup.find("table", {"class": "collDates"})
        table_rows = table_element.find_all_next("tr")

        row_index = 0
        for row in table_rows:
            if row_index < 1:
                row_index += 1
                continue
            else:
                details = row.find_all_next("td")
                dict_data = {
                    "type": details[1].get_text().replace("collection", "").strip(),
                    "collectionDate": datetime.strptime(
                        details[2].get_text(), "%A %d %B %Y"
                    ).strftime(date_format),
                }
                data["bins"].append(dict_data)
                row_index += 1

        return data
